/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is CSPapers.
 *
 * The Initial Developer of the Original Code is Bernhard Pollak.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */  

// preference service
var cspapers_prefService = {
    
	init : function() {
	  try {
      var aPrefFormat = nsPreferences.copyUnicharPref("extensions.cspapers.format", "");
      var aPrefEtAl = nsPreferences.copyUnicharPref("extensions.cspapers.etal", "");
      if (!aPrefFormat) aPrefFormat="%Y - %A - %T";
      if (!aPrefEtAl) aPrefEtAl="3";
      document.getElementById("cspapersPrefFormatTextbox").value=aPrefFormat;
      document.getElementById("cspapersPrefEtAlTextbox").value=aPrefEtAl;
    } catch(ex) {
      dump("ERROR [cspapers_prefService.init]"+ex+"\n");
    }
	},

	accept : function() {
	  try {
  	  var aPrefFormat=document.getElementById("cspapersPrefFormatTextbox").value;
      var aPrefEtAl=document.getElementById("cspapersPrefEtAlTextbox").value;
  	  if (aPrefFormat=="") {
  	    alert("The format string could not be empty!");
  	    return false;
  	  }
  	  if (aPrefEtAl=="") aPrefEtAl="0";
  	  if (parseInt(aPrefEtAl)==NaN) {  	  
  	    alert("The maximum authors should be a number!");
  	    return false;
  	  } 	    	   
  	  nsPreferences.setUnicharPref("extensions.cspapers.format", aPrefFormat);
  	  nsPreferences.setUnicharPref("extensions.cspapers.etal", aPrefEtAl);			
  	  return true;
  		
    } catch(ex) {
      dump("ERROR [cspapers_prefService.accept]"+ex+"\n");
    }	
    return false;			
	}
	
} 

// show a save dialog
function cspapers_showSaveAsDialog(aFileName)
{      
  // check autoDownload                   
  var downloadBranch=Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("browser.download.");  
  try {
    var downloadDir = downloadBranch.getComplexValue("downloadDir", Components.interfaces.nsILocalFile);
  } catch(ex) { }
  var useDownloadDir = downloadBranch.getBoolPref("useDownloadDir");
  if (useDownloadDir && downloadDir && downloadDir.exists()) {
    downloadDir.append(aFileName);
    return downloadDir.path;
  }    
  
  // show "Save File as" Dialog
	var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.init(window, "Save File as...", nsIFilePicker.modeSave);
  fp.defaultString = aFileName;
  fp.appendFilters(nsIFilePicker.filterAll);
  
  try {
    if (downloadDir.exists()) fp.displayDirectory = downloadDir;
  } catch (e) { }
  
  var res = fp.show();                                                         
  // selected "OK" or replace file "yes"
  if ((res == nsIFilePicker.returnOK) || (res == nsIFilePicker.returnReplace)) {   
    return fp.file.path;
  }                                  
  return null;
} 

  // download aSourceURL to aTargetFilename                              
  // (works also on local files...)
function cspapers_downloadFile(aSourceURL,aTargetFilename)
{               	    
  try {       
    dump("downloading file:\n-aSourceURL: "+aSourceURL+"\n-aTargetFilename: "+aTargetFilename+"\n");            

  	//new obj_URI object
    var obj_URI = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService).newURI(aSourceURL, null, null);  
         
    //if (obj_URI.isEmpty) return false;
         
    //new file object
    var obj_TargetFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile); 
    //set file with path      
    // NOTE: This function has a known bug on the macintosh and other OSes 
    // which do not represent file locations as paths. If you do use this 
    // function, be very aware of this problem!
    obj_TargetFile.initWithPath(aTargetFilename);       
    //if file doesn't exist, create
    //if (!obj_TargetFile.exists())
    //  obj_TargetFile.create(0x00,0644);                                
     
    //new persistence object
    var obj_Persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(Components.interfaces.nsIWebBrowserPersist);
	
		var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    obj_TargetURI=ioService.newFileURI(obj_TargetFile);

    try {
      // set up the transfer object for the download progress dialog
      var obj_Transfer = Components.classes["@mozilla.org/transfer;1"].createInstance(Components.interfaces.nsITransfer);    
      obj_Transfer.init(obj_URI,obj_TargetURI, "", null, null, null, obj_Persist);
    } catch(ex) {
      var obj_Transfer=null;
    }
    
    // set flags     
    const nsIWBP = Components.interfaces.nsIWebBrowserPersist; 
    var flags = nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES |
                nsIWBP.PERSIST_FLAGS_FROM_CACHE |
                nsIWBP.PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION;
                //nsIWBP.PERSIST_FLAGS_BYPASS_CACHE;        
    obj_Persist.persistFlags = flags; 
    // set progress listener (will show up in the download dialog)     
    obj_Persist.progressListener = obj_Transfer;
    //save file to target
    obj_Persist.saveURI(obj_URI,null,null,null,null,obj_TargetFile);                    
    
    return true;  
    
  } catch (ex) { 	  
    alert("CSPapers: Could not download the file!\n"+ex);    	
  }  
  return false;
}      

function cspapers_trim(aString) {
	try {
    return (aString.replace(/\s+$/,"").replace(/^\s+/,""));
  } catch(ex) {
      return aString;
  }
}  

	// replace illegal characters  
function cspapers_getValidFileName(aFileName)
{
  aFileName = aFileName.replace(/[\"\?!~`]+/g, "");       //" 
	aFileName = aFileName.replace(/[\\\/\|\;\*]+/g, "_");
	aFileName = aFileName.replace(/[%]+/g, "@");
	aFileName = aFileName.replace(/[\<]+/g, "(");
	aFileName = aFileName.replace(/[\>]+/g, ")");
	aFileName = aFileName.replace(/[\:]+/g, ";");	
	dump("cspapers_getValidFileName: "+aFileName+"\n");
	return aFileName;
}   

function cspapers_getShortName(aText,aBorder) {
  if (aText.length>aBorder) return aText.substring(0,aBorder)+"[...]";
  return aText;  
}

function cspapers_getEtAl(aAuthors,aEtAl) {
  var aResult=""; 
  if (aEtAl>0) { 
    if (aAuthors.length>aEtAl) {
      for (var i=0;i<aEtAl;i++) {
        aResult=aResult+aAuthors[i]+",";
      }
      aResult=aResult+aAuthors[aEtAl]+" et.al.";            
    }    
    if (aAuthors.join(",").length<=aResult.length) return aAuthors.join(",");
  }
  return aResult;
}

function cspapers_inspectNode(aNode,aCount) {
  try {
    if (aNode.style.backgroundColor!="") return;
    dump("cspapers_inspectNode: "+aNode.nodeName+" / "+aCount+"\n");
    if ((aCount % 210) < 70) {
      var r=255-(aCount % 70)-(aCount%2);
      var g=100-(aCount % 70)-(aCount%2);
      var b=100-(aCount % 70)-(aCount%2);
    } else if ((aCount % 150) < 140) {
      var r=100-(aCount % 70)-(aCount%2);
      var g=255-(aCount % 70)-(aCount%2);
      var b=100-(aCount % 70)-(aCount%2);
    } else {
      var r=100-(aCount % 70)-(aCount%2);
      var g=100-(aCount % 70)-(aCount%2);
      var b=255-(aCount % 70)-(aCount%2);
    }      
    aNode.style.backgroundColor = "rgb("+r+","+g+","+b+")"; 
  } catch (ex) {
  }
        
}

/* function cspapers_parseDOM(aNode,aDirection,aCount) {
  try {
    if (aNode==null) return aCount;
    var upNode=aNode;
    var downNode=aNode;
    var curNode=aNode;
    var aMode=true;
    dump("cspapers_parseDOM: "+aNode.nodeName+"\n");
    while ((upNode!=null) || (downNode!=null)) {          
      if (((aMode) || (downNode==null)) && (upNode!=null)) {
        curNode=upNode;
        upNode=upNode.previousSibling;
      }
      if (((!aMode) || (upNode==null)) && (downNode!=null)) {     
        downNode=downNode.nextSibling;
        if (downNode!=null) curNode=downNode; 
      }
      if ( curNode.nodeName != "#text" && curNode.nodeName != "#comment" ) {    
  	    cspapers_inspectNode(curNode,aCount++); 
  	    aCount=cspapers_parseDOM(curNode.firstChild,"down",aCount);
  	  }
      aMode=!aMode;                  
    }      
    if (aDirection!="down") aCount=cspapers_parseDOM(curNode.parentNode,aDirection,aCount);
    return aCount;
  } catch(ex) {
    dump("cspapers_parseDOM: "+ex+"\n");
  }    
}*/



function cspapers_generateNames(aBody,aCurLink) {
  dump("cspapers_generateNames called!\n");
  var aResult=new Object();
  aResult["fullname"]=new Array();
  aResult["menuname"]=new Array();
  try {    
  
    try {    
      var aPrefFormat = "";
      var aPrefEtAl = "";
      aPrefFormat = nsPreferences.copyUnicharPref("extensions.cspapers.format", "");
      aPrefEtAl = nsPreferences.copyUnicharPref("extensions.cspapers.etal", "");
    } catch(ex) {}   
    if (aPrefFormat=="") aPrefFormat = "%Y - %A - %T";
    if (aPrefEtAl=="") aPrefEtAl = "3";  

    var aTable=aBody.getElementsByTagName("table")[0];
    var i=1;
    while (aTable.getAttribute("class")!="h") {
    	var aTable=aBody.getElementsByTagName("table")[i]
    	i++;
    	if (i>5) break;
    }
    var aCenter=aTable.getElementsByTagName("center")[0];       
    var aTitle=aCenter.getElementsByTagName("span")[0].firstChild.firstChild.nodeValue;
    dump("aTitle found: "+aTitle+"\n");
    var aAuthors=aCenter.lastChild.nodeValue;
    dump("aAuthors found: "+aAuthors+"\n");
    var aYear= "";
    var re=new RegExp(/(\([0-9][0-9][0-9][0-9]\))/);
    try {
      if (aTitle.match(re)) {
        var aYear=RegExp.$1;        
        try {
          aTitle=cspapers_trim(aTitle.replace(aYear,""));                
          aYear=aYear.replace(/\(/, "");
          aYear=aYear.replace(/\)/, "");    
          dump("aYear found: "+aYear+"\n");
        } catch(ex) {
          aYear="";
        }
      }
    } catch(ex) {
      aTitle="";
      aYear="";
    }
    
    // authors        
    var aShortAuthors1=new Array();
    var aShortAuthors2=new Array();
    var aAuthor=[];            
    try {
      aAuthors=aAuthors.split(",");        
    } catch(ex) {
      aAuthors=[];
    }
    
    for (var i=0;i<aAuthors.length;i++) {
      aAuthors[i]=cspapers_trim(aAuthors[i]);
      aAuthor=aAuthors[i].split(" ");
      dump("processing aAuthor: "+aAuthors[i]+"\n");
      if (aAuthor.length==1) {
        aShortAuthors1.push(aAuthor[0]);
      } else if (aAuthor.length==2) {
        if ((aAuthor[0].length==2) && (aAuthor[0].indexOf(".")>-1)) {
          aShortAuthors1.push(aAuthor[1]);
        } else if ((aAuthor[1].length==2) && (aAuthor[1].indexOf(".")>-1)) {
          aShortAuthors1.push(aAuthor[0]);            
        } else {
          aShortAuthors1.push(aAuthor[1]);
          aShortAuthors2.push(aAuthor[0]);
        }
      } else if (aAuthor.length==3) {
        aShortAuthors1.push(aAuthor[aAuthor.length-1]);
        aShortAuthors2.push(aAuthor[0]);
      }        
    }
    
    var aAuthors1=cspapers_getEtAl(aShortAuthors1,aPrefEtAl);
    var aAuthors2=cspapers_getEtAl(aShortAuthors2,aPrefEtAl);
    var aAuthors3=cspapers_getEtAl(aAuthors,aPrefEtAl);
    
    if (aAuthors1=="")  {   
      for (var i=0;i<aShortAuthors1.length-1;i++) aAuthors1=aShortAuthors1[i]+","+aAuthors1;
      if (aShortAuthors1.length>0) aAuthors1=aAuthors1+aShortAuthors1[aShortAuthors1.length-1];
    }
    if (aAuthors2=="")  {   
      for (var i=0;i<aShortAuthors2.length-1;i++) aAuthors2=aShortAuthors2[i]+","+aAuthors2;
      if (aShortAuthors2.length>0) aAuthors2=aAuthors2+aShortAuthors2[aShortAuthors2.length-1];
    }
    if (aAuthors3=="")  {   
      for (var i=0;i<aAuthors.length-1;i++) aAuthors3=aAuthors[i]+","+aAuthors3;
      if (aAuthors.length>0) aAuthors3=aAuthors3+aAuthors[aAuthors.length-1];
    }      
                    
    var e=aCurLink.lastIndexOf(".");
    var aExt="";
    if ((e>-1) && (e>aCurLink.length-5)) aExt=aCurLink.substring(e,aCurLink.length);
    
    var aFileName = aPrefFormat.replace("%Y",aYear)+aExt;
    var aShortName = aFileName.replace("%T",cspapers_getShortName(aTitle,15));
    aFileName = aFileName.replace("%T",aTitle);
    aResult["fullname"].push(cspapers_getValidFileName(aFileName.replace("%A",aAuthors1)));
    aResult["menuname"].push(cspapers_getValidFileName(aShortName.replace("%A",cspapers_getShortName(aAuthors1,30))));                   
    if (aAuthors2!="") {
      aResult["fullname"].push(cspapers_getValidFileName(aFileName.replace("%A",aAuthors2)));
      aResult["menuname"].push(cspapers_getValidFileName(aShortName.replace("%A",cspapers_getShortName(aAuthors2,30))));
    }    
    if (aAuthors3!="") {
      aResult["fullname"].push(cspapers_getValidFileName(aFileName.replace("%A",aAuthors3)));
      aResult["menuname"].push(cspapers_getValidFileName(aShortName.replace("%A",cspapers_getShortName(aAuthors3,30))));
    }    
 
  } catch(ex) {
    dump("CSPapers: Could not generate the filename!\n"+ex);    	
  }  
  return aResult;
}

                   
function cspapers_menuHandler(aFileName) {
  var aCurLink=gContextMenu.link.href;      
  if (aCurLink) { 
    dump("cspapers_menuHandler: link found: "+aCurLink+"\n");
    dump("cspapers_menuHandler: aFileName: "+aFileName+"\n");

    var aDestFile = cspapers_showSaveAsDialog(aFileName);
    if (aDestFile) cspapers_downloadFile(aCurLink,aDestFile);
    
  }
}  

function cspapers_deleteMenuEntries(aMenu) {
  try {
    for (var i = aMenu.firstChild.childNodes.length - 1; i > -1; i--) {
      dump("cspapers_deleteMenuEntries: "+aMenu.firstChild.childNodes[i].id+"\n");
      aMenu.firstChild.removeChild(aMenu.firstChild.childNodes[i]);
    }
  } catch(ex) {
    dump("cspapers_deleteMenuEntries:"+ex+"\n");
  }      
}

function cspapers_addMenuEntries(aMenu,aItems) {
  for (var i=0; i<aItems["fullname"].length; i++) {
    dump("cspapers_addMenuEntries: "+aItems["fullname"][i]+"\n");
    var newItem = document.createElement("menuitem");
    newItem.setAttribute("id","cspapers-cmenu-item"+i);
    newItem.setAttribute("oncommand","cspapers_menuHandler(\""+aItems["fullname"][i]+"\");");     
    newItem.setAttribute("label",aItems["menuname"][i]); 
    aMenu.firstChild.appendChild(newItem);    
  }
}

function cspapers_updateMenuEntries(aMenu) {  
  try {
    cspapers_deleteMenuEntries(aMenu);
    var aCurLink=gContextMenu.link.href;      
    if (aCurLink) {
      dump("link found: "+aCurLink+"\n");
      //cspapers_parseDOM(gContextMenu.link,"up",0);
      
      var aDocument=top.window._content.document;
      var aBody=aDocument.getElementsByTagName("body")[0];
      if (aBody) {    
        dump("body found!\n");
        var aFileNames=cspapers_generateNames(aBody,aCurLink);      
        if (aFileNames["fullname"].length>0) {
          cspapers_addMenuEntries(aMenu,aFileNames);
          return true;
        } 
      }  
       
    }    
  } catch(ex) {
    dump("ERROR [cspapers_updateMenuEntries]"+ex+"\n");  
  }  
  return false;
} 

function cspapers_contextPopupShowing() {
  try {
    var menuitem = document.getElementById("cspapers-cmenu");
    if (menuitem) {      
      var aDocument=top.window._content.document;
     if (aDocument.location.href.indexOf("citeseer")==-1) {  
        dump("\n\ncspapers_contextPopupShowing: "+menuitem.id+"\n");
        //menuitem.hidden = true;
      } else {
        cspapers_updateMenuEntries(menuitem);
        menuitem.hidden = false;
      }
    }
  } catch(ex) {
    dump("ERROR [cspapers_contextPopupShowing]"+ex+"\n");  
  }
}

function cspapers_initOverlay() {
  var menu = document.getElementById("contentAreaContextMenu");
  menu.addEventListener("popupshowing", cspapers_contextPopupShowing, false);
}

window.addEventListener("load", cspapers_initOverlay, false);

